@@ -9,6 +9,8 @@
 #ifndef __XML_VERSION_H__
 #define __XML_VERSION_H__
 
+#include <libxml/xmlexports.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -18,7 +20,7 @@ extern "C" {
  * your library and includes mismatch
  */
 #ifndef LIBXML2_COMPILING_MSCCDEF
-extern void xmlCheckVersion(int version);
+XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
 #endif /* LIBXML2_COMPILING_MSCCDEF */
 
 /**
@@ -26,21 +28,21 @@ extern void xmlCheckVersion(int version);
  *
  * the version string like "1.2.3"
  */
-#define LIBXML_DOTTED_VERSION "2.5.11"
+#define LIBXML_DOTTED_VERSION "2.6.0"
 
 /**
  * LIBXML_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXML_VERSION 20511
+#define LIBXML_VERSION 20600
 
 /**
  * LIBXML_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXML_VERSION_STRING "20511"
+#define LIBXML_VERSION_STRING "20600"
 
 /**
  * LIBXML_TEST_VERSION:
@@ -48,7 +50,7 @@ extern void xmlCheckVersion(int version);
  * Macro to check that the libxml version in use is compatible with
  * the version the software has been compiled against
  */
-#define LIBXML_TEST_VERSION xmlCheckVersion(20511);
+#define LIBXML_TEST_VERSION xmlCheckVersion(20600);
 
 #ifndef VMS
 #if 0
@@ -81,6 +83,60 @@ extern void xmlCheckVersion(int version);
 #endif
 #endif
 
+/**
+ * LIBXML_TREE_ENABLED:
+ *
+ * Whether the DOM like tree manipulation API support is configured in
+ */
+#if 1
+#define LIBXML_TREE_ENABLED
+#endif
+
+/**
+ * LIBXML_OUTPUT_ENABLED:
+ *
+ * Whether the serialization/saving support is configured in
+ */
+#if 1
+#define LIBXML_OUTPUT_ENABLED
+#endif
+
+/**
+ * LIBXML_PUSH_ENABLED:
+ *
+ * Whether the push parsing interfaces are configured in
+ */
+#if 1
+#define LIBXML_PUSH_ENABLED
+#endif
+
+/**
+ * LIBXML_READER_ENABLED:
+ *
+ * Whether the xmlReader parsing interface is configured in
+ */
+#if 1
+#define LIBXML_READER_ENABLED
+#endif
+
+/**
+ * LIBXML_WRITER_ENABLED:
+ *
+ * Whether the xmlWriter saving interface is configured in
+ */
+#if 1
+#define LIBXML_WRITER_ENABLED
+#endif
+
+/**
+ * LIBXML_SAX1_ENABLED:
+ *
+ * Whether the older SAX1 interface is configured in
+ */
+#if 1
+#define LIBXML_SAX1_ENABLED
+#endif
+
 /**
  * LIBXML_FTP_ENABLED:
  *
@@ -99,6 +155,15 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_HTTP_ENABLED
 #endif
 
+/**
+ * LIBXML_VALID_ENABLED:
+ *
+ * Whether the DTD validation support is configured in
+ */
+#if 1
+#define LIBXML_VALID_ENABLED
+#endif
+
 /**
  * LIBXML_HTML_ENABLED:
  *
@@ -108,6 +173,15 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_HTML_ENABLED
 #endif
 
+/**
+ * LIBXML_LEGACY_ENABLED:
+ *
+ * Whether the deprecated APIs are compiled in for compatibility
+ */
+#if 1
+#define LIBXML_LEGACY_ENABLED
+#endif
+
 /**
  * LIBXML_C14N_ENABLED:
  *
@@ -199,7 +273,7 @@ extern void xmlCheckVersion(int version);
 #endif
 
 /**
- * LIBXML_UNICODE_ENABLED
+ * LIBXML_UNICODE_ENABLED:
  *
  * Whether the Unicode related interfaces are compiled in
  */
@@ -208,7 +282,7 @@ extern void xmlCheckVersion(int version);
 #endif
 
 /**
- * LIBXML_REGEXP_ENABLED
+ * LIBXML_REGEXP_ENABLED:
  *
  * Whether the regular expressions interfaces are compiled in
  */
@@ -217,7 +291,7 @@ extern void xmlCheckVersion(int version);
 #endif
 
 /**
- * LIBXML_AUTOMATA_ENABLED
+ * LIBXML_AUTOMATA_ENABLED:
  *
  * Whether the automata interfaces are compiled in
  */
@@ -226,7 +300,7 @@ extern void xmlCheckVersion(int version);
 #endif
 
 /**
- * LIBXML_SCHEMAS_ENABLED
+ * LIBXML_SCHEMAS_ENABLED:
  *
  * Whether the Schemas validation interfaces are compiled in
  */
@@ -234,29 +308,6 @@ extern void xmlCheckVersion(int version);
 #define LIBXML_SCHEMAS_ENABLED
 #endif
 
-/**
- * LIBXML_DLL_IMPORT:
- *
- * Used on Windows (MS C compiler only) to declare a variable as 
- * imported from the library. This macro should be empty when compiling
- * libxml itself. It should expand to __declspec(dllimport)
- * when the client code includes this header, and that only if the client
- * links dynamically against libxml.
- * For this to work, we need three macros. One tells us which compiler is
- * being used and luckily the compiler defines such a thing: _MSC_VER. The
- * second macro tells us if we are compiling libxml or the client code and
- * we define the macro IN_LIBXML on the compiler's command line for this 
- * purpose. The third macro, LIBXML_STATIC, must be defined by any client 
- * code which links against libxml statically. 
- */
-#ifndef LIBXML_DLL_IMPORT
-#if (defined(_MSC_VER) || defined(__BORLANDC__) || defined(__CYGWIN__)) && !defined(IN_LIBXML) && !defined(LIBXML_STATIC)
-#define LIBXML_DLL_IMPORT __declspec(dllimport)
-#else
-#define LIBXML_DLL_IMPORT
-#endif
-#endif
-
 /**
  * ATTRIBUTE_UNUSED:
  *
