@@ -18,9 +18,7 @@
 #endif
 #endif
 
-#if defined(WIN32) && !defined(__CYGWIN__) && !defined(__MINGW32__)
-#include "win32config.h"
-#elif defined(macintosh)
+#if defined(macintosh)
 #include "config-mac.h"
 #else
 #include "config.h"
@@ -46,4 +44,5 @@
  * function while checking if the callback exists.
  */
 extern int __xmlRegisterCallbacks;
+void __xmlIOErr(int domain, int code, const char *extra);
 #endif /* ! __XML_LIBXML_H__ */
