@@ -29,6 +29,7 @@
 #include <libxml/parser.h>
 #include <libxml/globals.h>
 
+#if defined(LIBXML_CATALOG_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
 static int shell = 0;
 static int sgml = 0;
 static int noout = 0;
@@ -39,7 +40,6 @@ static int convert = 0;
 static int verbose = 0;
 static char *filename;
 
-#ifdef LIBXML_CATALOG_ENABLED
 
 #ifndef XML_SGML_DEFAULT_CATALOG
 #define XML_SGML_DEFAULT_CATALOG "/etc/sgml/catalog"
@@ -580,8 +580,8 @@ int main(int argc, char **argv) {
     return(exit_value);
 }
 #else
-int main(int argc, char **argv) {
-    fprintf(stderr, "libxml was not compiled with catalog support\n");
+int main(int argc ATTRIBUTE_UNUSED, char **argv ATTRIBUTE_UNUSED) {
+    fprintf(stderr, "libxml was not compiled with catalog and output support\n");
     return(1);
 }
 #endif
